#!/bin/sh
#
PATHOLD=$PATH
################ Arguments ##################
ARCH=$1
ADDON=$2
#############################################

################## Paths ####################
CURRENT_PATH=`pwd`;

if [ "$ADDON" == "dist" ]; then
    DIST_DIR="/opt/dist_dir"
else
    DIST_DIR=""
fi

URL="test.com"

GCCDIR=""
PREFIX=""

LOCAL_GCCDIR="/usr/bin/"
LOCAL_PREFIX="/usr/local"

MINGW_GCCDIR="/usr/bin/"
MINGW_PREFIX="/usr/i686-mingw32/mingw"

TUXBOX_GCCDIR="/opt/tuxbox/bin"
TUXBOX_PREFIX="/opt/tuxbox/powerpc-tuxbox-linux-gnu"

oe7025_GCCDIR="/opt/oe7025/cross/bin"
oe7025_PREFIX="/opt/oe7025/staging/mipsel-linux"
oe7025_PATH="/opt/oe7025"

STM_GCCDIR="/opt/STM/STLinux-2.0/devkit/sh4/bin"
STM_PREFIX="/opt/STM/STLinux-2.0/devkit/sh4/target"

openwrt_GCCDIR="/opt/openwrt/bin"
openwrt_PREFIX="/opt/openwrt/mipsel-linux"

cris_GCCDIR="/opt/cris/bin"
cris_PREFIX="/opt/cris/cris-axis-linux-gnu"

nslu2_GCCDIR="/opt/nslu2/cross/bin"
nslu2_PREFIX="/opt/nslu2/staging/armeb-linux"

CONFIGURED=".configured"

function configure_local {
    CHOST=`gcc -dumpmachine`;
    SUFFIX=".$(gcc -dumpmachine | cut -f1 -d"-")"
    GCCDIR=$LOCAL_GCCDIR
    PREFIX=$LOCAL_PREFIX
    LDFLAGS="-ldl -lpthread"
    CFLAGS="-Os -Wall"
}

function configure_win32 {
    CHOST="i686-mingw32"
    SUFFIX=".win32"
    GCCDIR=$MINGW_GCCDIR
    PREFIX=$MINGW_PREFIX
}

function configure_openwrt {
    CHOST="mipsel-linux-uclibc"
    SUFFIX=".openwrt"
    GCCDIR=$openwrt_GCCDIR
    PREFIX=$openwrt_PREFIX
}

function configure_nslu2 {
    CHOST="armeb-linux"
    SUFFIX=".nslu2"
    GCCDIR=$nslu2_GCCDIR
    PREFIX=$nslu2_PREFIX
    CFLAGS="-g -Os -Wall -I$nslu2_PREFIX/include -L$nslu2_PREFIX/lib"
}

function configure_cris {
    CHOST="cris-axis-linux-gnu"
    SUFFIX=".cris"
    GCCDIR=$cris_GCCDIR
    PREFIX=$cris_PREFIX
}

function configure_dm7025 {
    CHOST="mipsel-linux"
    SUFFIX=".mips"
    GCCDIR=$oe7025_GCCDIR
    PREFIX=$oe7025_PREFIX
    LDFLAGS="-ldl -lpthread"
    CFLAGS="-g -Os -Wall -I$oe7025_PATH/staging/mipsel-linux/include"
}

function configure_dm7020 {
    CHOST="powerpc-linux"
}

function configure_tuxbox {
    CHOST="powerpc-tuxbox-linux-gnu"
    SUFFIX=".ppc"
    GCCDIR=$TUXBOX_GCCDIR
    PREFIX=$TUXBOX_PREFIX
    LDFLAGS="-ldl -lpthread"
    CFLAGS="-g -Os -Wall"
}

function configure_stm {
    CHOST="sh4-linux"
    SUFFIX=".st40"
    GCCDIR=$STM_GCCDIR
    PREFIX=$STM_PREFIX
    LDFLAGS="-ldl -lpthread"
    CFLAGS="-g -Os -Wall -I$STM_PREFIX/include"
}

function eend()
{
    local retval=${1:-0}
    shift
    return ${retval}
}

function copylib {
	if [ -f $PREFIX/lib/libfeynman.so.0.0.0 ]; then
		cp $PREFIX/lib/libfeynman.so.0.0.0 $DIST_DIR/libfeynman.so.0$SUFFIX
		echo "Copied lib to $DIST_DIR/libfeynman.so.0$SUFFIX"

		SIZEB=$(ls -la $DIST_DIR/libfeynman.so.0$SUFFIX | cut -f5 -d" ")
		SIZE=$(echo "$SIZEB/1024" | bc)
		echo $SIZE
		MD5=$(md5sum $DIST_DIR/libfeynman.so.0$SUFFIX | cut -f1 -d" ")
		echo "7http://$URL/libfeynman.so.0$SUFFIX ($SIZE KB)[$MD5] is a fresh build from cvs.." >>/tmp/buildlog
		chmod 777 /tmp/buildlog
	else
		eend $? "Cant find lib.." || exit $?
	fi
}

function build {
    if [ "$GCCDIR" != "" ]; then
        export PATH=$PATH:$GCCDIR
    else
        echo "Compiler not defined"
        exit
    fi

    if [ "$PREFIX" == "" ]; then
        echo "prefix not defined"
        exit
    fi

    CONFIGURED="$CONFIGURED.$CHOST"

    if [ ! -f $CONFIGURED ]; then
	if [ -f .configured* ]; then
		rm .configured*
	fi
        rm -R $CURRENT_PATH/$CHOST
        if [ ! -d $CURRENT_PATH/$CHOST ]; then
            mkdir $CURRENT_PATH/$CHOST
            eend $? "Cannot create workdir" || exit $?
        fi
        cd $CURRENT_PATH/$CHOST
        echo "-----------------------------------------------------"
        echo "Configurating feynman for $CHOST"
        echo "-----------------------------------------------------"
        if [ -f .configured* ]; then
            rm .configured*
        fi
        CFLAGS=$CFLAGS LDFLAGS=$LDFLAGS ../configure --host=$CHOST --prefix=$PREFIX --libdir=$PREFIX/lib --with-ssl=cyassl
        eend $? "Cannot configure" || exit $?
        touch $CURRENT_PATH/$CONFIGURED
    fi

    cd $CURRENT_PATH/$CHOST
    if [ "$CLEAN" != "no" ]; then
        echo "-----------------------------------------------------"
        echo "Cleaning feynman for $CHOST"
        echo "-----------------------------------------------------"
        make clean
        eend $? "Cannot clean" || exit $?
    fi

    echo "-----------------------------------------------------"
    echo "Building feynman for $CHOST"
    echo "-----------------------------------------------------"
    make
    eend $? "Cannot make" || exit $?

    echo "-----------------------------------------------------"
    echo "Installing feynman for $CHOST"
    echo "-----------------------------------------------------"
    make install
    eend $? "Cannot make install" || exit $?
}

if [ "$ARCH" = "dm7025" ]; then
    configure_dm7025

elif [ "$ARCH" = "dm7020" ]; then
    configure_dm7020

elif [ "$ARCH" = "tuxbox" ]; then
    configure_tuxbox

elif [ "$ARCH" = "openwrt" ]; then
    configure_openwrt

elif [ "$ARCH" = "nslu2" ]; then
    configure_nslu2

elif [ "$ARCH" = "cris" ]; then
    configure_cris

elif [ "$ARCH" = "win32" ]; then
    configure_win32

elif [ "$ARCH" = "local" ]; then
    configure_local

elif [ "$ARCH" = "kathrein" ]; then
    configure_stm

else
    echo "$ARCH is not supported by this script..."
    exit
fi

build

if [ "$ADDON" == "dist" ]; then
    copylib
fi

export PATH=$PATHOLD
