/***************************************************************************
 *   Copyright (C) 2007 by Dr. Feynman                                     *
 *   dr@feynman.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>

#include "feynman.h"

t_feynman_emm *feynman_add_remote_emm(t_feynman_remote_server *server)
{
    if(!server) return NULL;

    t_feynman_emm *emm = (t_feynman_emm *)malloc(sizeof(t_feynman_emm));

    emm->caid = 0;
    emm->provid = 0;
    emm->address = 0;
    emm->type= 0;
    emm->emm = NULL;
    emm->emm_len = 0;
    emm->card_id = 0;
    emm->session = 0;
    gettimeofday(&emm->time_added, NULL);

    emm->next = server->emm_db;
    server->emm_db = emm;

    return emm;
}

t_feynman_emm *feynman_add_local_emm(t_feynman_local_server *server)
{
    t_feynman_emm *emm = (t_feynman_emm *)malloc(sizeof(t_feynman_emm));

    emm->caid = 0;
    emm->provid = 0;
    emm->address= 0;
    emm->type= 0;
    emm->emm = NULL;
    emm->emm_len = 0;
    emm->card_id = 0;
    emm->session = 0;
    gettimeofday(&emm->time_added, NULL);

    //Allow emms without a server, remember to free them
    if(server != NULL)
    {
        emm->next = server->emm_db;
        server->emm_db = emm;
    }

    return emm;
}

 void feynman_set_emm_data(t_feynman_emm *emm, unsigned char *emm_data, int emm_len)
{
    if(!emm) return;
    if(!emm_data) return;
    if(emm_data == emm->emm) return;
    if(emm->emm != NULL) free(emm->emm);

    emm->emm = malloc(emm_len);
    emm->emm_len = emm_len;
    memcpy(emm->emm, emm_data, emm_len);
}

void feynman_free_emm(t_feynman_emm *emm)
{
    if(!emm) return;

    if(emm->emm != NULL)
    {
        free(emm->emm);
    }
    free(emm);
}

t_feynman_emm *feynman_get_remote_emm(t_feynman_remote_server *server, int session, int card_id)
{
    t_feynman_emm *emm;

    if(!server) return NULL;

    for(emm=server->emm_db; emm!=NULL; emm=emm->next)
    {
        if(emm->session == session && emm->card_id == card_id)
        {
            return emm;
        }
    }

    return NULL;
}

t_feynman_emm *feynman_get_local_emm(t_feynman_local_server *server, int session, int card_id)
{
    t_feynman_emm *emm;

    if(!server) return NULL;

    for(emm=server->emm_db; emm!=NULL; emm=emm->next)
    {
        if(emm->session == session && emm->card_id == card_id)
        {
            return emm;
        }
    }

    return NULL;
}

void feynman_del_remote_emm(t_feynman_remote_server *server, t_feynman_emm *emm_to_del)
{
    if(!server) return;
    if(!emm_to_del) return;

    t_feynman_emm *emm = server->emm_db;
    t_feynman_emm *prev = server->emm_db;
    while(emm != NULL)
    {
        t_feynman_emm *tmp_emm = emm->next;
        if(emm_to_del == emm)
        {
            if(prev != NULL)
            {
                prev->next = emm->next;
            }
            if(server->emm_db == emm)
            {
                server->emm_db = emm->next;
            }
            feynman_free_emm(emm);
            return;
        }
        prev = emm;
        emm = tmp_emm;
    }
}

void feynman_del_remote_emms(t_feynman_remote_server *server)
{
    if(!server) return;

    t_feynman_emm *emm = server->emm_db;
    t_feynman_emm *prev = server->emm_db;
    while(emm != NULL)
    {
        t_feynman_emm *tmp_emm = emm->next;
        if(1)
        {
            if(prev != NULL)
            {
                prev->next = emm->next;
            }
            if(server->emm_db == emm)
            {
                server->emm_db = emm->next;
            }
            feynman_free_emm(emm);
        }
        prev = emm;
        emm = tmp_emm;
    }
}

void feynman_del_local_emm(t_feynman_local_server *server, t_feynman_emm *emm_to_del)
{
    if(!server) return;
    if(!emm_to_del) return;

    t_feynman_emm *emm = server->emm_db;
    t_feynman_emm *prev = server->emm_db;
    while(emm != NULL)
    {
        t_feynman_emm *tmp_emm = emm->next;
        if(emm_to_del == emm)
        {
            if(prev != NULL)
            {
                prev->next = emm->next;
            }
            if(server->emm_db == emm)
            {
                server->emm_db = emm->next;
            }
            feynman_free_emm(emm);
            return;
        }
        prev = emm;
        emm = tmp_emm;
    }
}

void feynman_del_local_emms(t_feynman_local_server *server)
{
    if(!server) return;

    t_feynman_emm *emm = server->emm_db;
    t_feynman_emm *prev = server->emm_db;
    while(emm != NULL)
    {
        t_feynman_emm *tmp_emm = emm->next;
        if(1)
        {
            if(prev != NULL)
            {
                prev->next = emm->next;
            }
            if(server->emm_db == emm)
            {
                server->emm_db = emm->next;
            }
            feynman_free_emm(emm);
        }
        prev = emm;
        emm = tmp_emm;
    }
}
